/*
 * Copyright (c) 2016      Mellanox Technologies, Inc.
 *                         All rights reserved.
 * Copyright (c) 2018      IBM Corporation.  All rights reserved.
 * Copyright (c) 2018      Intel, Inc. All rights reserved.
 * Copyright (c) 2019      Research Organization for Information Science
 *                         and Technology (RIST).  All rights reserved.
 * Copyright (c) 2021      Nanook Consulting  All rights reserved.
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */


#ifndef PRTE_VERSION_H
#define PRTE_VERSION_H

/* define PRTE version */
#define PRTE_VERSION_MAJOR 3L
#define PRTE_VERSION_MINOR 0L
#define PRTE_VERSION_RELEASE 14L

#define PRTE_NUMERIC_VERSION 0x0003000e
#endif
